
var DigitalFolioInstallHelper = (function () {    
    var beingUninstalled = false;
    
    var listener = {
        onInstalling: function (addon, needsRestart) {            
            if (addon.id == DigitalFolioInstallHelper.addonId)
                DigitalFolioOverlay.prefs.deleteBranch("");  
        },       
        onUninstalling: function (addon) {            
            if (addon.id == DigitalFolioInstallHelper.addonId)
                beingUninstalled = true;

        },
        onOperationCancelled: function (addon) {
            if (addon.id == DigitalFolioInstallHelper.addonId) {
                beingUninstalled = (addon.pendingOperations & AddonManager.PENDING_UNINSTALL) != 0;
            }
        }

    };

    return {
        addonId: "{793700EF-E9B4-4C47-B207-B0F45F69F4B7}",

        observe: function (subject, topic, data) {
            if (topic == "em-action-requested") {

                subject.QueryInterface(Components.interfaces.nsIUpdateItem);
                if (subject.id == addonId) {
                    if (data == "item-installed") {
                        DigitalFolioOverlay.prefs.deleteBranch("");   
                    }
                    if (data == "item-uninstalled") {
                        beingUninstalled = true;
                    }
                    else if (data == "item-cancel-action") {
                        beingUninstalled = false;
                    }
                }
            }
            else if (topic == "quit-application-granted") {                
                if (beingUninstalled) {
                    DigitalFolioOverlay.prefs.deleteBranch("");                
                }
                this.unregister();
            }
        },

        register: function () {
            if (!("@mozilla.org/extensions/manager;1" in Components.classes)) {
                Components.utils.import("resource://gre/modules/AddonManager.jsm");
            } 

            if(AddonManager)
                AddonManager.addAddonListener(listener);

            var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
            observerService.addObserver(this, "em-action-requested", false);
            observerService.addObserver(this, "quit-application-granted", false);
        },

        unregister: function () {
            if (AddonManager)
                AddonManager.removeAddonListener(listener);

            var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
            observerService.removeObserver(this, "em-action-requested");
            observerService.removeObserver(this, "quit-application-granted");
        }
    };

})();

var DigitalFolioOverlay = {
    mainWindow: null,
    prefs: Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.DigitalFolio."),

    onLoad: function () {
        this.strings = document.getElementById("DigitalFolio-strings");

        mainWindow = window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                    .getInterface(Components.interfaces.nsIWebNavigation)
                    .QueryInterface(Components.interfaces.nsIDocShellTreeItem)
                    .rootTreeItem
                    .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                    .getInterface(Components.interfaces.nsIDOMWindow);

        DigitalFolioInstallHelper.register();

        mainWindow.gBrowser.addEventListener("DOMContentLoaded", function (e) { DigitalFolioOverlay.onUserPageLoad(e); }, true);

        DigitalFolioOverlay.doFirstRun();
    },

    onUserPageLoad: function (e) {
        DigitalFolioOverlay.subscribeToEvents();
        var mainBrowserDoc = mainWindow.gBrowser.contentDocument;

        if (mainBrowserDoc) {
            var sidebarComm = mainBrowserDoc.getElementById('DigitalFolioSidebarComm');

            if (sidebarComm) {
                var mainBrowserWindow = mainWindow.gBrowser.contentWindow;                
                mainBrowserWindow.wrappedJSObject.DigitalFolioFFExt = 1;                
            }
        }

    },

    subscribeToEvents: function () {
        var mainBrowserDoc = mainWindow.gBrowser.contentDocument;
        mainBrowserDoc.addEventListener('ShowSidebar', function (e) { DigitalFolioOverlay.showSideBarHandler(e); });
        mainBrowserDoc.addEventListener('ChangeSidebarUrl', function (e) { DigitalFolioOverlay.changeSidebarUrlHandler(e); });
    },

    showSideBarHandler: function (e) {
        //Application.console.log('showSideBarHandler');
        toggleSidebar('viewSidebar_DigitalFolio', true);
    },

    addToFolioClicked: function (triggerNode) {
        var url = mainWindow.gBrowser.contentDocument.location.href;
        //Application.console.log(triggerNode.tagName);
        if (triggerNode.tagName.toLowerCase() == 'a') {
            url = triggerNode.href;
            //Application.console.log(url);
        }

        toggleSidebar('viewSidebar_DigitalFolio', true);
        var sidebarWindow = document.getElementById("sidebar").contentWindow;

        if (sidebarWindow.location.href == "chrome://digitalfolio/content/ff-sidebar.xul") {
            sidebarWindow.DigitalFolio.notifySideBarBrowser.UrlDropped(url);
            return;
        }

        window.setTimeout(function () {
            sidebarWindow.DigitalFolio.notifySideBarBrowser.UrlDropped(url);
        }, 3000);

    },

    changeSidebarUrlHandler: function (e) {
        var url = e.target.getAttribute("sidebarUrl");

        toggleSidebar('viewSidebar_DigitalFolio', true);
        var sidebarWindow = document.getElementById("sidebar").contentWindow;

        if (sidebarWindow.location.href == "chrome://digitalfolio/content/ff-sidebar.xul") {
            sidebarWindow.DigitalFolio.ChangeSideBarUrl(url);
            return;
        }

        window.setTimeout(function () {
            sidebarWindow.DigitalFolio.ChangeSideBarUrl(url);
        }, 1000);

    },

    addToolbarButton: function (buttonId, locationPreferences) {

        if (!document.getElementById(buttonId)) {

            var toolbar = document.getElementById("nav-bar");

            var currentSet = toolbar.currentSet;
            var newSet = currentSet;
            var setItems = currentSet.split(',');

            var toolbox = document.getElementById("navigator-toolbox");
            var toolboxDocument = toolbox.ownerDocument;

            function getIndex(array, val) {
                for (var i = 0; i < array.length; i++) {
                    if (array[i] == val) {
                        return i;
                    }
                }

                return -1;
            }

            var added = false;

            for (var i = 0, _len = locationPreferences.length; i < _len; i++) {
                var before = false;
                var item = locationPreferences[i];

                if (item.indexOf(":before") != -1) {
                    item = item.replace(":before", "");
                    before = true;
                }

                if (getIndex(setItems, item) != -1) {
                    if (before) {
                        newSet = currentSet.replace(item, buttonId + "," + item);
                    }
                    else {
                        newSet = currentSet.replace(item, item + "," + buttonId);
                    }

                    added = true;
                    break;
                }
            }

            if (!added) {
                newSet = toolbar.currentSet + "," + buttonId;
            }

            toolbar.currentSet = newSet;
            toolbar.setAttribute("currentset", newSet);

            toolboxDocument.persist(toolbar.id, "currentset");

            try {
                BrowserToolboxCustomizeDone(true);
            } catch (e) { }
        }

    },

    getVersion: function (callback) {
        var addonId = DigitalFolioInstallHelper.addonId;

        if ("@mozilla.org/extensions/manager;1" in Components.classes) {
            // < Firefox 4
            var version = Components.classes["@mozilla.org/extensions/manager;1"]
				.getService(Components.interfaces.nsIExtensionManager).getItemForID(addonId).version;

            callback(version);
        }
        else {
            // Firefox 4.
            Components.utils.import("resource://gre/modules/AddonManager.jsm");

            AddonManager.getAddonByID(addonId, function (addon) {
                callback(addon.version);
            });
        }
    },

    doFirstRun: function () {
        function isMajorUpdate(version1, version2) {
            if (!version1) {
                return true;
            }
            else {

                var oldParts = version1.split(".");
                var newParts = version2.split(".");

                if (newParts[0] != oldParts[0] || newParts[1] != oldParts[1] || newParts[2] != oldParts[2]) {
                    return true;
                }
            }

            return false;
        }

        function firstRun(version) {
            if (isMajorUpdate(DigitalFolioOverlay.prefs.getCharPref("lastVersion"), version)) {
                DigitalFolioOverlay.addToolbarButton("DigitalFolio-toolbar-button", ["bookmarks-menu-button-container", "home-button", "search-container", "urlbar-container"]);

                toggleSidebar('viewSidebar_DigitalFolio', true);

                window.setTimeout(function () {
                    //gBrowser.selectedTab = gBrowser.addTab("http://www.digitalfolio.com/Register/");

                    var mainWindow = window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                        .getInterface(Components.interfaces.nsIWebNavigation)
                        .QueryInterface(Components.interfaces.nsIDocShellTreeItem)
                        .rootTreeItem
                        .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                        .getInterface(Components.interfaces.nsIDOMWindow);

                    var mainBrowserDoc = mainWindow.gBrowser.contentDocument;
                    mainBrowserDoc.location.href = "http://www.digitalfolio.com/Register/";
                }, 1500); //Firefox 2 fix - or else tab will get closed


                DigitalFolioOverlay.prefs.setCharPref("lastVersion", version);
            }

        }

        DigitalFolioOverlay.getVersion(firstRun);
    }
};

window.addEventListener("load", DigitalFolioOverlay.onLoad, false);


